@@ -177,7 +177,12 @@ bayessurvreg2 <- function
                              rhob       = as.double(0),
                              rho.accept = as.integer(0),
                              rhobI      = as.integer(0),
-                             rhobD      = as.double(0),            
+                             rhobD      = as.double(0),
+                             mclass.sens.spec = as.double(0),
+                             mclass.logtime   = as.double(0),
+                             mclass.status    = as.integer(0),
+                             mclass.paramI    = as.integer(c(0, 0, 0, rep(0, nobs), 0, 0)),
+                             mclass.paramD    = as.double(c(1, 1, 1, 1)),
                              iter = as.integer(prinit$iter),
                              nsimul = as.integer(nsimul.run1),
                              store = as.integer(storeV),
@@ -231,6 +236,11 @@ bayessurvreg2 <- function
                              rho.accept = as.integer(0),
                              rhobI      = as.integer(0),
                              rhobD      = as.double(0),
+                             mclass.sens.spec = as.double(0),
+                             mclass.logtime   = as.double(0),
+                             mclass.status    = as.integer(0),
+                             mclass.paramI    = as.integer(c(0, 0, 0, rep(0, nobs), 0, 0)),
+                             mclass.paramD    = as.double(c(1, 1, 1, 1)),
                              iter = as.integer(fit$iter),
                              nsimul = as.integer(nsimul.run2),
                              store = as.integer(fit$store),
